//
//  ckTimeZoneListCluster.m
//  CKs Time Zone Picker
//
//  Created by C.K. Haun on 3/27/14.
//  Copyright (c) 2014 Ravenware Industries LLC. All rights reserved.
//
// Classes to manage what I call time zone clusters and bundles.
// These two classes deconstruct the standard system time zone names
// and organize them into clusters.  Each cluster is a region containing the names of the
// time zone cities in that region.  This is handy for the UIPickerControl, and other things

#import "ckTimeZoneListBundle.h"

@implementation ckTimeZoneListCluster
@synthesize header;
@synthesize elements;

// the first one in
-(id)initWithElement:(ckTimeZoneListElement *)starterElement{
    self.header = [starterElement.prefix copy];
    self.elements=[[NSMutableArray alloc]initWithObjects:starterElement  , nil];
    return self;
}


// take an element and see if it belongs in my cluster, based on the prefix, or region, name
-(BOOL)mergeIntoCluster:(ckTimeZoneListElement *)element{
    BOOL retVal = NO;
    if([element.prefix isEqualToString:self.header]){
        [self.elements addObject:element];
        retVal=YES;
    }
    return (retVal);
}

// is this zone in this cluster?
// negative means we did not find it
-(NSInteger) elementInCluster:(ckTimeZoneListElement*)element{
    NSInteger retVal = 0;
    for(ckTimeZoneListElement *eddy in elements){
        if([element.completeName isEqualToString:eddy.completeName]){
            break;
        }
        retVal++;
    }
    if(retVal >= [elements count]){
        retVal = -1;
    }
    return (retVal);
}

- (void)encodeWithCoder:(NSCoder *)encoder 
{
    [encoder encodeObject:self.header forKey:@"header"];
    [encoder encodeObject:self.elements forKey:@"elements"];
}

- (id)initWithCoder:(NSCoder *)decoder 
{
    self = [super init];
    if (self) {
        self.header = [decoder decodeObjectForKey:@"header"];
        self.elements = [decoder decodeObjectForKey:@"elements"];
    }
    return self;
}

@end

// The bundle is the whole set of time zones, arranges into clusters
// Clusters are 'Region:all cities in region'
// this makes it easy to parse in the UIPicker.

@implementation ckTimeZoneListBundle : NSObject
@synthesize clusters;
@synthesize elements;
-(id)init
{
    BOOL merged = NO;
    // ask the system for all the time zone names it knows about.
    NSArray * zones=[NSTimeZone knownTimeZoneNames];   
    elements = [[NSMutableArray alloc]init];
    clusters = [[NSMutableArray alloc]init];
    // for my own sake I wanted the standard US time zones and abbreviations.
    // you might want to remove this.
// add the standard US time zones
    NSString *usName = @"U.S. Standard";
    NSString *names[]={
        @"EST",
        @"EDT",
        @"CST",    
        @"CDT",
        @"MST",
        @"MDT",
        @"PST",
        @"PDT",
        @"AST",
        @"ADT",
        @"HST",
        @"HDT",
    };
    int usUTCOffsets[]={
        -5,
        -4,
        -6,
        -5,
        -7,
        -6,
        -8,
        -7,
        -9,
        -8,
        -10,
        -9
    };
    int qq;  // Loop variables have been 'qq' since 1979.
    for(qq=0;qq < (sizeof(usUTCOffsets)/sizeof(int));qq++){
        ckTimeZoneListElement *temp=[[ckTimeZoneListElement alloc]initAndSythesizeElement:usName newName:names[qq] offsetFromUTC:usUTCOffsets[qq]];
        merged=NO;
        // merge this element into an existing cluster
        for(ckTimeZoneListCluster *cluster in clusters){
            merged = [cluster mergeIntoCluster:temp];
            if(merged)break;
        }
        if(merged== NO){
            // no cluster for this one.   Create it
            ckTimeZoneListCluster *newOne = [[ckTimeZoneListCluster alloc]initWithElement:temp];
            if(newOne)[clusters addObject:newOne];
        }

    }
// create elements for each name provided for us by the System
int index=0;
for(NSString *name in zones){
    ckTimeZoneListElement *temp = [[ckTimeZoneListElement alloc] initWithName: name andIndex:index ];
    [elements addObject:temp];
    index++;
}
// walk through all the elements and merge them into the cluster they belong to
    merged = NO;
    for(ckTimeZoneListElement *ed in elements){
        for(ckTimeZoneListCluster *cluster in clusters){
            merged = [cluster mergeIntoCluster:ed];
            if(merged)break;
        }
        if(merged== NO){
            // no line for this one.   Create it
            ckTimeZoneListCluster *newOne = [[ckTimeZoneListCluster alloc]initWithElement:ed];
            if(newOne)[clusters addObject:newOne];
        }
    }

    
return (self);
}

// negative means we did not find this element aywhere
-(NSInteger)inWhichCluster:(ckTimeZoneListElement *)element{
    NSString *tempHeader = element.prefix;
    NSInteger retVal = 0;
    for(ckTimeZoneListCluster *tommy in clusters){
        if([tommy.header isEqualToString:tempHeader]){
            break;
        }
        retVal ++;
    }
    // sanity check
    if (retVal >= [clusters count]) {
        retVal = -1;
        
    }
    return (retVal);
}

- (void)encodeWithCoder:(NSCoder *)encoder 
{
    [encoder encodeObject:self.clusters forKey:@"clusters"];
    [encoder encodeObject:self.elements forKey:@"elements"];
}

- (id)initWithCoder:(NSCoder *)decoder 
{
    self = [super init];
    if (self) {
        self.clusters = [decoder decodeObjectForKey:@"clusters"];
        self.elements = [decoder decodeObjectForKey:@"elements"];
    }
    return self;
}

// Get one of our custom clusters from a system time zone object
-(ckTimeZoneListElement *)getElementForNSTimeZone:(NSTimeZone *)theZone{
    ckTimeZoneListElement *temp=nil;
    
    NSString *theName = [theZone name];
    // find the name in the list
    for(ckTimeZoneListCluster *tommy in clusters){
        for(ckTimeZoneListElement *bobby in tommy.elements){
            if([bobby.completeName isEqualToString:theName]){
                temp = bobby;
                break;
            }
            if(temp !=nil)break;
        }
        
    }
        
        
        return(temp);
        
        }

-(ckTimeZoneListElement *)getElementForTimeZoneName:(NSString *)theZone{
    ckTimeZoneListElement *retVal=nil;
    
    NSString *theName =theZone    ;
    
    // find the name in the list
    for(ckTimeZoneListCluster *thisCluster in clusters){
        for(ckTimeZoneListElement *thisElement in thisCluster.elements){
            if([thisElement.completeName isEqualToString:theName]){
                retVal = thisElement;
                break;
            }
            if(retVal !=nil)break;
        }
        
    }
    
    
    return(retVal);
    
}



@end